'''
作者: 小鱼
公众号: 鱼香ROS
QQ交流群: 2642868461
描述: file content
'''
import sys
import argparse
import rclpy
try:
    import transforms3d as tfs
except:
    print("请先安装transforms3d..")

from fishrotate.RoateAnimation import RoateAnimation


def main(args=None):
    # if args is None:
        # args = sys.argv[1:]
    
    parse = argparse.ArgumentParser(description='fishrotate:基于ROS2的坐标旋转动画工具\r\n使用样例:fishrotate rxyz base rotate 0.0 0.0 0.0 45.0 60.0 90.0')

    rotate_types = ''
    for rotate_type in tfs.euler._AXES2TUPLE.keys():
        rotate_types += f'{rotate_type},'

    parse.add_argument("rotate_order",default="rxyz",type=str,help=f'旋转顺序：{rotate_types[:-1]},默认:rxyz')
    parse.add_argument("origin",default="base",type=str,help=f'原坐标系名字，默认:base')
    parse.add_argument("rotate",default="rotate",type=str,help=f'旋转的坐标系名字,默认:rotate')
    parse.add_argument("x",default="0.0",type=float,help=f'rotate沿origin的X轴平移,默认:0.0')
    parse.add_argument("y",default="0.0",type=float,help=f'rotate沿origin的Y轴平移,默认:0.0')
    parse.add_argument("z",default="0.0",type=float,help=f'rotate沿origin的Z轴平移,默认:0.0')


    parse.add_argument("r1",default="0.0",type=float,help=f'绕第1个转轴的旋转,默认:45.0')
    parse.add_argument("r2",default="0.0",type=float,help=f'绕第2个转轴的旋转,默认:60.0')
    parse.add_argument("r3",default="0.0",type=float,help=f'绕第3个转轴的旋转,默认:90.0')


    parse.add_argument("speed",default="10.0",type=float,help=f'旋转速度，度/秒,默认:10.0')

    args = parse.parse_args()
    print(f"发布{args.origin}到{args.rotate}的动画，平移:{args.x},{args.y},{args.z},旋转:{args.rotate_order}:{args.r1},{args.r2},{args.r3} 速度:{args.speed}")

    rclpy.init()
    rotate = RoateAnimation(div=args.speed/100.0)
    rotate.add_transform(args.origin,
                args.rotate,
                [args.x,args.y,args.z],
                [args.r1,args.r2,args.r3],
                order=args.rotate_order
                )
    rclpy.spin(rotate)





'''
作者: 小鱼
公众号: 鱼香ROS
QQ交流群: 2642868461
描述: RoateAnimation类
'''
from threading import Thread
from queue import Queue
import transforms3d as tfs
from tf2_ros import TransformBroadcaster
from geometry_msgs.msg import TransformStamped
from rclpy.node import Node
import rclpy
import math


class RoateAnimation(Node):
    def __init__(self,name="roatate",div=0.001):
        super().__init__(name)
        self.queue = Queue()
        self.trans_frames = {}
        self.broadcaster = TransformBroadcaster(self)
        self.rate = self.create_rate(100)
        self.thread = Thread(name="t_thread",target=self.transform_thread)
        self.thread.start()
        self.div = div

        
    def add_transform(self,parent,name,tran=[0.0,0.0,0.0],rotate=[0.0,0.0,0.0],order="sxyz"):
        if len(rotate)==3:
            for i in range(3):
                rotate[i] = math.radians(rotate[i])
        self.trans_frames[name] = {"name":name,"parent":parent,
                                   "target_rotate":rotate,"target_tran":tran,
                                   "current_rotate":[0.0,0.0,0.0],"current_tran":[0.0,0.0,0.0],
                                   "order":order}
        
        
    def destory_all(self):
        """
        取消所有的旋转变换
        """
        self.trans_frames.clear()
    
    def _get_transform_msg(self,parent,name,tran=[0.0,0.0,0.0],rotate=[0.0,0.0,0.0],order="sxyz"):
        t = TransformStamped()
        t.header.stamp = self.get_clock().now().to_msg()
        t.header.frame_id = parent
        t.child_frame_id = name
        t.transform.translation.x = tran[0]
        t.transform.translation.y = tran[1]
        t.transform.translation.z = tran[2]
        if len(rotate)==3:
            rotate = tfs.euler.euler2quat(rotate[0],rotate[1],rotate[2],order)
            t.transform.rotation.x = rotate[1]
            t.transform.rotation.y = rotate[2] 
            t.transform.rotation.z = rotate[3]
            t.transform.rotation.w = rotate[0]
        return t
    
    def _tick_once(self):
        for name in self.trans_frames.keys():
            if self.trans_frames[name]["current_tran"][0]<self.trans_frames[name]["target_tran"][0]:
                self.trans_frames[name]["current_tran"][0] += self.div
            elif self.trans_frames[name]["current_tran"][1]<self.trans_frames[name]["target_tran"][1]:
                self.trans_frames[name]["current_tran"][1] += self.div
            elif self.trans_frames[name]["current_tran"][2]<self.trans_frames[name]["target_tran"][2]:
                self.trans_frames[name]["current_tran"][2] += self.div
                
            elif self.trans_frames[name]["current_rotate"][0]<self.trans_frames[name]["target_rotate"][0]:
                self.trans_frames[name]["current_rotate"][0] += self.div    
            elif self.trans_frames[name]["current_rotate"][1]<self.trans_frames[name]["target_rotate"][1]:
                self.trans_frames[name]["current_rotate"][1] += self.div                
            elif self.trans_frames[name]["current_rotate"][2]<self.trans_frames[name]["target_rotate"][2]:
                self.trans_frames[name]["current_rotate"][2] += self.div 
            else:
                self.trans_frames[name]["current_rotate"] = [0.0,0.0,0.0]
                self.trans_frames[name]["current_tran"] = [0.0,0.0,0.0]
            
    def transform_thread(self):
        while rclpy.ok():
            self._tick_once()
            for name in self.trans_frames.keys():
                msg = self._get_transform_msg(self.trans_frames[name]['parent'],self.trans_frames[name]['name'],
                                                self.trans_frames[name]['current_tran'],
                                                self.trans_frames[name]['current_rotate'],
                                                self.trans_frames[name]['order'])
                self.broadcaster.sendTransform(msg)
            self.rate.sleep()


from setuptools import setup

package_name = 'fishrotate'

setup(
    name=package_name,
    version='0.1.0',
    packages=[package_name],
    data_files=[
        ('share/ament_index/resource_index/packages',
            ['resource/' + package_name]),
        ('share/' + package_name, ['package.xml']),
    ],
    install_requires=['setuptools','transforms3d'],
    zip_safe=True,
    maintainer='fishros',
    maintainer_email='fishros@foxmail.com',
    description='基于ROS2旋转可视化工具',
    license='OSI Approved :: BSD License',
    tests_require=['pytest'],
    entry_points={
        'console_scripts': [
             'fishrotate = fishrotate.fishrotate:main',
        ],
    },
)
